/*--------------------------------------------------------------------*/
/*--- Instrument IR to perform memory checking operations.         ---*/
/*---                                               fc_translate.h ---*/
/*--------------------------------------------------------------------*/

/*
   This file is part of FlowCheck, a heavyweight Valgrind tool for
   detecting leakage of secret information.

   Based on MemCheck, a heavyweight Valgrind tool for detecting memory
   errors.

   Copyright (C) 2000-2007 Julian Seward 
      jseward@acm.org

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.

   The GNU General Public License is contained in the file COPYING.
*/

typedef struct _TraceEnv TraceEnv;

/* Carries around state during flowcheck instrumentation. */
typedef
   struct _FCEnv {
      /* MODIFIED: the superblock being constructed.  IRStmts are
         added. */
      IRSB* bb;

      /* MODIFIED: a table [0 .. #temps_in_original_bb-1] which maps
         original temps to their current their current shadow temp.
         Initially all entries are IRTemp_INVALID.  Entries are added
         lazily since many original temps are not used due to
         optimisation prior to instrumentation.  Note that floating
         point original tmps are shadowed by integer tmps of the same
         size, and Bit-typed original tmps are shadowed by the type
         Ity_I8.  See comment below. */
      IRTemp* tmpMap;
      Int     n_originalTmps; /* for range checking */

      IRTemp enclosureModeTmp8;
      IRTemp enclosureModeTmp32;

      IRTemp rollbackModeTmp1;
      IRTemp rollbackModeTmp32;

      /* MODIFIED: indicates whether "bogus" literals have so far been
         found.  Starts off False, and may change to True. */
      Bool    bogusLiterals;

      /* READONLY: the guest layout.  This indicates which parts of
         the guest state should be regarded as 'always defined'. */
      VexGuestLayout* layout;

      /* READONLY: the host word type.  Needed for constructing
         arguments of type 'HWord' to be passed to helper functions.
         Ity_I32 or Ity_I64 only. */
      IRType hWordTy;

      TraceEnv *trace_env;
   }
   FCEnv;

typedef  IRExpr  IRAtom;

IRExpr* expr2vbits ( struct _FCEnv* fce, IRExpr* e );
Bool isShadowAtom ( FCEnv* fce, IRAtom* a1 );
IRTemp findShadowTmp ( FCEnv* fce, IRTemp orig );

/* The offset of the shadow area, within the guest state. */
Int FC_(shadow_guest_offset);

/* Offset of the tags shadows within the shadow guest state. */
Int FC_(tags_guest_offset);
